local addon = MissionMinder:CreateAddon("MissionMinder-Garrison")
local module = addon:CreateModule("listView")

---------------------------------------------
-- INITIALIZE
---------------------------------------------
function module.initialize()
	local missionFrame =  GarrisonMissionFrame
	local missionList = missionFrame.MissionTab.MissionList

	-- "View Completed Missions" button will open "In Progress" tab instead of "CompleteMission" frame
	local viewButton = missionList.CompleteDialog.BorderFrame.ViewButton;
	viewButton.OriginalOnClick = viewButton:GetScript("OnClick")
	viewButton:SetScript("OnClick", MissionMinder_CompleteDialog_ShowInProgressMissionList);

	-- Override CompleteDialog initialize so we can target a specific mission
	--instead of defaulting to "first mission in list"
	missionFrame.OriginalOnClickViewCompletedMissionsButton = missionFrame.OnClickViewCompletedMissionsButton
	missionFrame.OnClickViewCompletedMissionsButton = MissionMinder_MissionFrame_ViewSpecificCompletedMission

	-- Completed Missions that haven't been looted are not considered "in progress" anymore
	--and need to be added to the list
	missionList.UpdateMissions = MissionMinder_InProgressMissionList_IncludeCompletedMissions
	missionList.OnUpdate = missionListOnUpdate_IncludeCompletedMissions
	missionList:SetScript("OnUpdate", missionList.OnUpdate)

	-- Sort InProgress missions by completed first
	Garrison_SortMissions = function(missions)
		table.sort(missions, MissionMinder_CompletedMissionsSortComparator);
	end

	-- Highlight completed missions
	hooksecurefunc(missionList, "Update", MissionMinder_InProgressMissionList_HighlightCompletedMissions);
	--GarrisonMissionFrame.MissionTab.MissionList.listScroll.update = GarrisonMissionList_Update; -- Set the update method to the new hooked method

	-- Clicking a completed mission will view it
	local missionButtons = missionList.listScroll.buttons;
	for i = 1, #missionButtons do
		local button = missionButtons[i];
		button:HookScript("OnClick", MissionMinder_InProgressMissionList_ViewMission);
	end

	-- Show special tooltips for completed missions
	--[[GarrisonMissionButton_SetInProgressTooltip = inprogressTooltips]]

	-- Create SuccessChance text
	for i = 1, #missionButtons do
		local button = missionButtons[i];
		local successChance = button:CreateFontString(nil, "ARTWORK", "GameFontNormalLarge");
		successChance:SetPoint("TOPLEFT", button.Title, "BOTTOMLEFT", 0, -17);
		successChance:SetTextColor(0, 1, 0);
		successChance:SetSize(115, 15)
		button.SuccessChance = successChance;
	end

	-- Create Party member frames
	for i = 1, #missionButtons do
		local button = missionButtons[i];
		button.Party = {}
		local partyMember = CreateFrame("Frame", nil, button, "MissionListPartyMemberPortrait")
		partyMember:SetPoint("LEFT", button.SuccessChance, "RIGHT", 20, 0);
		tinsert(button.Party, partyMember)
	end

	-- rewrite "Next" mission button as "Close"
	options:On("ListViewEnabled", function(option, value)
		local nextMissionButton = missionFrame.MissionComplete.NextMissionButton;
		nextMissionButton:SetText( value and CLOSE or NEXT )
	end)

	-- Clicking "Next" misison button closes the mission complete frame
	local nextMissionButton = GarrisonMissionFrame.MissionComplete.NextMissionButton
	nextMissionButton.OriginalOnClick = nextMissionButton:GetScript("OnClick")
	nextMissionButton:SetScript("OnClick", MissionMinder_MissionComplete_NextButton_CloseMissionComplete);

	-- Refresh completed missions list on GarrisonMissionFrame.MissionComplete:Hide
	--[[local events = MissionMinder.events;
	events:RegisterEvent("listview", "GARRISON_MISSION_COMPLETE_RESPONSE", refreshCompletedMissionsForQuickLoot);
	events:RegisterEvent("listview", "GARRISON_MISSION_BONUS_ROLL_COMPLETE", refreshCompletedMissionsForQuickLoot);
	GarrisonMissionFrame.MissionComplete:HookScript("OnHide", refreshCompletedMissions);

	-- Closing GarrisonMissionFrame should select the "Available Missions" tab. This will cause the next open to show
	--the completed missions dialog again (if applicable)
	GarrisonMissionFrame:HookScript("OnHide", selectAvailableMissions);

	-- Mission finished event should not show complete dialog if "In Progress" tab already selected
	-- Currently uses a pre-hook, with more work might be able to remove the pre-hook requirement and use secure post-hook
	addon.GarrisonMissionFrame_CheckCompleteMissions = GarrisonMissionFrame_CheckCompleteMissions;
	GarrisonMissionFrame_CheckCompleteMissions = showMissionCompleteFrame;]]
end
