local MAJOR, MINOR = "LibItemInfo-1.0", 0
local core = LibStub:NewLibrary(MAJOR, MINOR)
if not core then return end -- Already loaded



local function errorhandler(err)
	return geterrorhandler()(err)
end

-- We use a runtime compiled function so we can give it a specifc name in the callstack
local dispatcher_code = [[
	local xpcall, unpack, errorhandler = ...

	return function(self, ...)
	    local args = { ... }
		local callback = nil
		local function invoke() callback(unpack(args)) end

		for i = 1, #self.callbacks do
			callback = self.callbacks[i]
			xpcall(invoke, errorhandler)
		end
	end
]]
local dispatcher = assert(loadstring(dispatcher_code, "safecall LibItemInfo:Dispatch"))(xpcall, unpack, errorhandler)



local Callbacks = {
	New = function(self)
		local obj = {
			callbacks = {}
		}
		setmetatable(obj, self)
		return obj
	end;

	Register = function(self, callback)
		table.insert(self.callbacks, callback)
	end;

	Dispatch = dispatcher; -- runtime compiled function
}
Callbacks.__index = Callbacks

local callbacksByItem = {}


function core:GetItemInfo(itemID, callback)
	local info = { GetItemInfo(itemID) }

	-- Check to see if the item info is already available.
	if ( info[1] ) then
		callback(unpack(info))
		return true
	else
		self:OnItemInfoReceived(itemID, callback)
		return false
	end
end


function core:OnItemInfoReceived(itemID, callback)
	if ( not callbacksByItem[itemID] ) then
		callbacksByItem[itemID] = Callbacks:New()
	end
	callbacksByItem[itemID]:Register(callback);
end


local frame = CreateFrame("FRAME")
frame:SetScript("OnEvent", function(event, itemID)
	if ( event == "GET_ITEM_INFO_RECEIVED" ) then
		local callbacks = callbacksByItem[itemID]
		callbacksByItem[itemID] = nil
		if ( callbacks ) then
			callbacks:Dispatch(GetItemInfo(itemID))
		end
	end
end)
frame:RegisterEvent("GET_ITEM_INFO_RECEIVED")
frame:Hide()
