local MAJOR, MINOR = "LibSavedVariables-1.0", 0
local lib = LibStub:NewLibrary(MAJOR, MINOR)
if not lib then return end -- Already loaded

local function DEBUG(shouldLog, ...)
    if ( shouldLog ) then
        print(...)
    end
end

local OPTIONS = {
    Version = function(self, version, f)
        self.dataUpgrade.debug = self.debug
        self.dataUpgrade:Version(version, f)
    end;

    Load = function(self)
        self.dataUpgrade.debug = self.debug
        _G[self.savedVariable] = self.dataUpgrade:Upgrade(_G[self.savedVariable])
        self.initialized = true

        -- Fire all registered handlers
        for option, _ in pairs(self.callbacks.events) do
            self:Fire(option, self:GetOption(option))
        end
    end;

    SetDefaults = function(self)
        self.dataUpgrade.debug = self.debug
        _G[self.savedVariable] = self.dataUpgrade:Defaults()
        self.initialized = true
    end;

    On = function(self, option, func)
        -- Use GetTime() to ensure uniqueness with millisecond precision.
        self.registry.RegisterCallback(tostring(GetTime()), option, func)

        -- Fire callbacks immediately if already initialized
        if ( self.initialized ) then
            self:Fire(option, self:GetOption(option))
        end
    end;

    Fire = function(self, option, value)
        assert(self.initialized, "LibSavedVariables not initialized (Load() must be called)")
        DEBUG(self.debug, "LibSavedVariables:Fire() ", option, value)
        self.callbacks:Fire(option, value)
    end;

    Map = function(self, key, value)
        local parts = {}
        for i in value:gmatch("[^.]+") do
            table.insert(parts, i)
        end
        self.map[key] = parts
    end;

    GetOption = function(self, option)
        assert(self.initialized, "LibSavedVariables not initialized (Load() must be called)")
        assert(option, "Option cannot be nil")
        assert(self.map[option], "Unmapped option " .. option)

        local map = self.map[option]
        local options = _G[self.savedVariable]
        if ( options ) then
            for i = 1, #map do
                options = options[map[i]]
            end
        end
        return options
    end;

    SetOption = function(self, option, value)
        assert(self.initialized, "LibSavedVariables not initialized (Load() must be called)")
        assert(option, "Option cannot be nil")
        assert(self.map[option], "Unmapped option " .. option)

        local map = self.map[option]
        local options = _G[self.savedVariable]
        for i = 1, #map-1 do
            options = options[map[i]]
        end
        options[map[#map]] = value
        self:Fire(option, value)
    end;
}
OPTIONS.__index = OPTIONS

function lib:New(variable, versionProperty)
    local registry = {}
    local obj = {
        dataUpgrade = LibStub("LibDataUpgrade-1.0"):New(versionProperty);
        savedVariable = variable;
        map = {};

        registry = registry;
        callbacks = LibStub("CallbackHandler-1.0"):New(registry);

        initialized = false;
        debug = false;
    }
    setmetatable(obj, OPTIONS)
    return obj;
end
