local Lib = MissionMinder;
Lib.callbacks= Lib.callbacks or LibStub("CallbackHandler-1.0"):New(Lib)

local options = MissionMinder.options;
local L = LibStub("AceLocale-3.0"):GetLocale("MissionMinder");
local DEBUG = MissionMinder.debug;

MissionMinder.blizzfix = {};
local module = MissionMinder.blizzfix;

local function hidePlacerFrame()
	if ( GarrisonFollowerPlacerFrame:IsShown() ) then
		GarrisonFollowerPlacerFrame:Hide();
	end
end

local function GenericHybridScrollMouseOverUpdate(self, ...)
	local buttons = self.buttons;
	for i = 1, #buttons do
		if ( buttons[i]:IsShown() and buttons[i]:IsMouseOver() ) then
			local func = buttons[i]:GetScript("OnEnter");
			if(func) then
				func(buttons[i]);
			end
			return;
		end
	end
end

local function FollowerListScrollMouseOverUpdate(self, ...)
	GarrisonFollowerAbilityTooltip:Hide()
	GarrisonMissionMechanicFollowerCounterTooltip:Hide()

	local frame = nil
	local buttons = self.buttons
	for i = 1, #buttons do
		if ( buttons[i]:IsShown() and buttons[i]:IsMouseOver() ) then
			frame = buttons[i]

			local counters = buttons[i].Counters;
			for c = 1, #counters do
				if ( counters[c]:IsShown() and counters[c]:IsMouseOver() ) then
					frame = counters[c]
					break
				end
			end
			break
		end
	end

	if ( frame ) then
		local func = frame:GetScript("OnEnter")
		if(func) then
			func(frame)
		end
	end
end

local function hideFollowerListContextMenu(self, ...)
	GarrisonFollowerOptionDropDown.followerID = nil;
	CloseDropDownMenus();
end

local function PreloadMissionModels(...)
	local missionID = ...;
	if ( C_Garrison.GetMissionDisplayIDs(missionID) == nil ) then
		return; -- If no values returned, then return without preloading
	end

	module.MissionCompletePreload_LoadMission(...);
end

GarrisonLandingPageReportList_UpdateMouseOverTooltip = function(self)
	local buttons = self.buttons;
	for i = 1, #buttons do
		if ( buttons[i]:IsShown() and buttons[i]:IsMouseOver() ) then
			GarrisonLandingPageReportMission_OnEnter(buttons[i]);
			break;
		end
	end
end

local function showHideMissionPageEnvironment(mission)
	local self = GarrisonMissionFrame.MissionTab.MissionPage

	local isVisible = (self.environment ~= nil)
	self.Stage.MissionEnvIcon:SetShown(isVisible)
	self.Stage.MissionEnv:SetShown(isVisible)
end

local function initialize()
	DEBUG("   |cffffffffBlizzardFixes|r initialized");

	-- For added measure clearMouse now hides PlacerFrame if it somehow happens to be visible on accident
	hooksecurefunc("GarrisonMissionFrame_ClearMouse", hidePlacerFrame);


	-- Scrolling through various lists do not update the tooltip or context menus
	local missionScrollFunc = GarrisonMissionFrame.MissionTab.MissionList.listScroll:GetScript("OnMouseWheel")
	GarrisonMissionFrame.MissionTab.MissionList.listScroll:SetScript("OnMouseWheel", function(self, ...)
		if(not GarrisonMissionFrame.MissionTab.MissionList.CompleteDialog:IsShown()
				and not GarrisonMissionFrame.MissionComplete:IsShown()) then
			missionScrollFunc(self, ...)
		end
	end);
	GarrisonMissionFrame.FollowerList.listScroll:HookScript("OnMouseWheel", FollowerListScrollMouseOverUpdate);
	GarrisonMissionFrame.FollowerList.listScroll:HookScript("OnMouseWheel", hideFollowerListContextMenu);
	GarrisonLandingPage.FollowerList.listScroll:HookScript("OnMouseWheel", GenericHybridScrollMouseOverUpdate);

	-- C_Garrison.GetMissionDisplayIDs(missionID) will return nil in certain scenarios, and the blizzard code
	--does not handle nils. This case seems to occur when models have already been preloaded, and we try to
	--preload a second time.
	module.MissionCompletePreload_LoadMission = MissionCompletePreload_LoadMission;
	MissionCompletePreload_LoadMission = PreloadMissionModels;

	-- #55 : Scrolling the landing page causes nil lua error
	GarrisonLandingPageReport.List.listScroll:SetScript("OnMouseWheel", function(self, ...) HybridScrollFrame_OnMouseWheel(self, ...); GarrisonLandingPageReportList_UpdateMouseOverTooltip(self); end);

	-- #63 : Mission Rewards and CompleteDialog are shown at the same level causing Reward tooltips to show
	local CompleteDialog = GarrisonMissionFrame.MissionTab.MissionList.CompleteDialog
	CompleteDialog:SetFrameLevel(CompleteDialog:GetFrameLevel() + 5)

	-- #66 : Some missions don't have an environment so we should hide it
	hooksecurefunc("GarrisonMissionPage_ShowMission", showHideMissionPageEnvironment)
end

local events = Lib.events;
events:RegisterEvent("blizzfixes", "ADDON_LOADED", initialize);
