local addon, options, L, ext = unpack(select(2, ...))


---------------------------------------------
-- Extensions
---------------------------------------------
function ext.DEBUG(...)
	if ( MissionMinder.debug ) then
		print(...)
	end
end

function ext.DEBUGF(...)
	if ( MissionMinder.debug ) then
		print(string.format(...))
	end
end

local DEBUG = ext.DEBUG
local DEBUGF = ext.DEBUGF

local SECONDS = 1
local MINUTES = 60 * SECONDS
local HOURS = 60 * MINUTES
local DAYS = 24 * HOURS
ext.TIMEUNITS = { SECONDS, MINUTES, HOURS, DAYS }

local TIMES = { DAYS, HOURS, MINUTES}
local TIMESTRS = {INT_SPELL_DURATION_DAYS, INT_SPELL_DURATION_HOURS, INT_SPELL_DURATION_MIN }
function ext.secondsToString(seconds)
	local str = ""

	if  ( seconds < MINUTES ) then
		str = str .. string.format(INT_SPELL_DURATION_SEC, seconds);
	else
		local reportOn=2
		for i=1, #TIMES do
			if ( seconds >= TIMES[i] ) then
				if(reportOn <= 0) then
					break;
				end

				if ( str:len() > 0 ) then
					str = str .. " ";
				end

				str = str .. string.format(TIMESTRS[i], seconds / TIMES[i])
				seconds = seconds % TIMES[i]
				reportOn = reportOn - 1
			end
		end
	end

	return str
end


function ext.tfind(table, value, comparator)
	if ( not comparator ) then
		comparator = function(obj, value) return obj == value end
	elseif ( type(comparator) == "string" ) then
		local key = comparator
		comparator = function(obj, value) return obj[key] == value end
	end

	for k, v in pairs(table) do
		if ( comparator(v, value) ) then
			return v, k
		end
	end

	return nil
end

function ext.tmap(table, func)
	assert(func, "Mapping a table requires a function")
	if ( type(func) == "string" ) then
		local key = func
		func = function(k, v) return k, v[key] end
	end

	local mapped = {}
	for k, v in pairs(table) do
		local key, value = func(k, v)
		mapped[key] = value
	end
	return mapped
end

function ext.treduce(table, func)
	assert(func, "Reducing a table requires a function")

	local reduced = nil;
	if ( table ) then
		for k, v in pairs(table) do
			reduced = func(k, v, reduced)
		end
	end
	return reduced
end

function ext.foreach(table, func)
	for k,v in pairs(table) do
		func(k, v)
	end
end


function ext.initializeListScrollHook(frame, initMethod, func)
    if ( frame.listScroll.buttons and #frame.listScroll.buttons > 0 ) then
         ext.foreach(frame.listScroll.buttons, func)
    else -- FollowerList has not been initialized yet, hook into :Setup
        hooksecurefunc(frame, initMethod, function()
	         ext.foreach(frame.listScroll.buttons, func)
		end)
    end
end
