local addon, options, L, ext = unpack(select(2, ...))
local module = addon:CreateModule("options")

local DEBUG = ext.DEBUG
local DEBUGF = ext.DEBUGF

---------------------------------------------
-- Locale
---------------------------------------------
local L_CONFIGURE_MISSIONMINDER = L["Configure MissionMinder"]
local L_DEBUG = L["Debug"]
local L_ADDON_DESCRIPTION = L["Manage garrison missions in a more streamline, less-invasive manner"]
local L_LISTVIEW = L["List View"]
local L_LISTVIEW_DESCRIPTION = L["View completed missions as a list from the \"In Progress\" tab"]
local L_ENABLED = L["Enabled"]
local L_LISTVIEW_ENABLED_DESCRIPTION = L["View completed missions as a list from the \"In Progress\" tab"]
local L_QUICKLOOT = L["Quick loot"]
local L_QUICKLOOT_DESCRIPTION = L["Disable the completion frame entirely, skip all the fanfare, and just give me the loot already!"]
local L_SHOWXP = L["Show follower XP gains"]
local L_SHOWXP_DESCRIPTION = L["Show messages regarding individual follower XP gains"]
local L_GENERAL_OPTIONS = L["General Options"]
local L_SKIPFIGHTANIMATIONS = L["Skip fight animations"]
local L_SKIPFIGHTANIMATIONS_DESCRIPTION = L["Skip all fight animations and go straight to rewards"]
local L_AUTOOPENREWARDS = L["Automatically open rewards chest"]
local L_AUTOOPENREWARDS_DESCRIPTION = L["Remove the need to click on the rewards chest to recieve the rewards"]
local L_AUTOCLOSE = L["Automatically close on loot"]
local L_AUTOCLOSE_DESCRIPTION = L["Automatically close the completion dialog if all actions have been completed for a mission"]
local L_PLANNER = L["Planner"];
local L_SHOWFOLLOWERCOUNTERTOTALS = L["Show active follower counter totals"];
local L_SHOWFOLLOWERCOUNTERTOTALS_DESCRIPTION = L["Easily track how many of each counter you currently have active"]
local L_IGNOREXPGAINTRAITS = L["Ignore XP increase traits"]
local L_IGNOREXPGAINTRAITS_DESCRIPTION = L["Do not show XP increase traits on missions that do not reward Bonus XP"]


---------------------------------------------
-- OPTIONS
---------------------------------------------
options:Map("Debug", "debug")
options:Map("ListViewEnabled", "ListView.enabled")
options:Map("QuickLoot", "ListView.quickLoot")
options:Map("ShowFollowerXPGains", "ListView.showFollowerXPGains")
options:Map("SkipFightAnimations", "MissionComplete.skipFightAnimations")
options:Map("AutoOpenRewards", "MissionComplete.autoOpenRewards")
options:Map("AutoClose", "MissionComplete.autoClose")
options:Map("IgnoreXPGainTraits", "MissionPlanner.ignoreXPGainsTraits")
options:Map("ShowFollowerCounterTotals", "Followers.showFollowerCounterTotals")
options:Map("SummaryReport:ShowGarrisonMissions", "SummaryReport.Garrison.showMissions")
options:Map("SummaryReport:ShowGarrisonWorkOrders", "SummaryReport.Garrison.showWorkOrders")
options:Map("SummaryReport:ShowGarrisonBuildings", "SummaryReport.Garrison.showBuildings")
options:Map("SummaryReport:ShowGarrisonCache", "SummaryReport.Garrison.showCache")
options:Map("SummaryReport:ShowOrderHallMissions", "SummaryReport.OrderHall.showMissions")
options:Map("SummaryReport:ShowOrderHallWorkOrders", "SummaryReport.OrderHall.showWorkOrders")
options:Map("SummaryReport:ShowOrderHallTalents", "SummaryReport.OrderHall.showTalents")
options:Map("SummaryReport:ShowOrderHallTroops", "SummaryReport.OrderHall.showTroops")

options:Version("1.1", {
	debug = false;
	ListView = {
		enabled = true;
		quickLoot = false;
		showFollowerXPGains = false;
	};
	MissionComplete = {
		skipFightAnimations = true;
		autoOpenRewards = true;
		autoClose = false;
	};
	MissionPlanner = {
		ignoreXPGainsTraits = true;
	};
	Followers = {
		showFollowerCounterTotals = true;
	};
	SummaryReport = {
		Garrison = {
			showMissions = true;
			showWorkOrders = true;
			showBuildings = true;
			showCache = true;
		};
		OrderHall = {
			showMissions = true;
			showWorkOrders = true;
			showTalents = true;
			showTroops = true;
		};
	};
})

local function getOption(info)
	return options:GetOption(info.arg)
end

local function setOption(info, value)
	return options:SetOption(info.arg, value)
end

function options.ListViewEnabled()
	return options:GetOption("ListViewEnabled");
end

function options.AutoLootRewards()
	return options:GetOption("AutoOpenRewards");
end

function options.AutoClose()
	return options:GetOption("AutoClose");
end

function options.SkipFightAnimations()
	return options:GetOption("SkipFightAnimations");
end

function options.QuickLoot()
	return options.ListViewEnabled() and options:GetOption("QuickLoot");
end

function options.ShowFollowerXPGains()
	return options.QuickLoot() and options:GetOption("ShowFollowerXPGains");
end

function options.ShowFollowerCounterTotals()
	return options:GetOption("ShowFollowerCounterTotals");
end

function options.IgnoreXPGainTraits()
	return options:GetOption("IgnoreXPGainTraits");
end


function options.ShowGarrisonMissionsSummary()
	return options:GetOption("SummaryReport:ShowGarrisonMissions")
end

function options.ShowOrderHallMissionsSummary()
	return options:GetOption("SummaryReport:ShowOrderHallMissions")
end

function options.ShowMissionsSummary(garrType)
	local tbl = {
		[LE_GARRISON_TYPE_6_0] = options.ShowGarrisonMissionsSummary;
		[LE_GARRISON_TYPE_7_0] = options.ShowOrderHallMissionsSummary;
	}
	local func = tbl[garrType]

	if ( func ) then return func() end
	return false
end


function options.ShowGarrisonWorkOrdersSummary()
	return options:GetOption("SummaryReport:ShowGarrisonWorkOrders")
end

function options.ShowOrderHallWorkOrdersSummary()
	return options:GetOption("SummaryReport:ShowOrderHallWorkOrders")
end

function options.ShowWorkOrdersSummary(garrType)
	local tbl = {
		[LE_GARRISON_TYPE_6_0] = options.ShowGarrisonWorkOrdersSummary;
		[LE_GARRISON_TYPE_7_0] = options.ShowOrderHallWorkOrdersSummary;
	}
	local func = tbl[garrType]

	if ( func ) then return func() end
	return false
end


function options.ShowGarrisonBuildingsSummary()
	return options:GetOption("SummaryReport:ShowGarrisonBuildings")
end

function options.ShowBuildingsSummary(garrType)
	local tbl = {
		[LE_GARRISON_TYPE_6_0] = options.ShowGarrisonBuildingsSummary;
	}
	local func = tbl[garrType]

	if ( func ) then return func() end
	return false
end


function options.ShowOrderHallTalentsSummary()
	return options:GetOption("SummaryReport:ShowOrderHallTalents")
end

function options.ShowTalentsSummary(garrType)
	local tbl = {
		[LE_GARRISON_TYPE_7_0] = options.ShowOrderHallTalentsSummary;
	}
	local func = tbl[garrType]

	if ( func ) then return func() end
	return false
end


function options.ShowOrderHallTroopsSummary()
	return options:GetOption("SummaryReport:ShowOrderHallTroops")
end

function options.ShowTroopsSummary(garrType)
	local tbl = {
		[LE_GARRISON_TYPE_7_0] = options.ShowOrderHallTroopsSummary;
	}
	local func = tbl[garrType]

	if ( func ) then return func() end
	return false
end


function options.ShowGarrisonCacheSummary()
	return options:GetOption("SummaryReport:ShowGarrisonCache")
end



---------------------------------------------
-- OPTIONS PANE
---------------------------------------------
local configName = string.format("%sOptionsPane", addon.name);
LibStub("AceConfig-3.0"):RegisterOptionsTable(configName, {
	name = addon.name  .. " " .. addon.version,
	desc = L_ADDON_DESCRIPTION,
	type = "group",
	args = {
		desc = {
			order = 1,
			type = "description",
			name = L_ADDON_DESCRIPTION,
			width = "full"
		};
		emptySpace = {
			order = 3,
			type = "description",
			name = " ",
			width = "full"
		};
		listview = {
			order = 10,
			type = "group",
			inline = true,
			name = L_LISTVIEW,
			args = {
				description = {
					order = 1,
					type = "description",
					name = L_LISTVIEW_DESCRIPTION,
					width = "full",
				};
				spacer = {
					order = 2,
					type = "description",
					name = " ",
					width = "full",
				};
				listviewEnabled = {
					order = 10,
					name = L_ENABLED,
					desc = L_LISTVIEW_ENABLED_DESCRIPTION,
					type = "toggle",
					width = "full",
					set = setOption,
					get = getOption,
					arg = "ListViewEnabled"
				};
				quickLoot = {
					order = 25,
					name = L_QUICKLOOT,
					desc = L_QUICKLOOT_DESCRIPTION,
					disabled = function() return not options.ListViewEnabled(); end,
					type = "toggle",
					width = "full",
					set = setOption,
					get = getOption,
					arg = "QuickLoot"
				};
				showXP = {
					order = 30,
					name = L_SHOWXP,
					desc = L_SHOWXP_DESCRIPTION,
					disabled = function() return not options.ListViewEnabled() or not options.QuickLoot(); end,
					type = "toggle",
					width = "full",
					set = setOption,
					get = getOption,
					arg = "ShowFollowerXPGains"
				};
			}
		};
		general = {
			order = 15,
			type = "group",
			inline = true,
			disabled = MissionMinder.QuickLoot,
			name = L_GENERAL_OPTIONS,
			args = {
				skipFightAnimations = {
					order = 5,
					name = L_SKIPFIGHTANIMATIONS,
					desc = L_SKIPFIGHTANIMATIONS_DESCRIPTION,
					type = "toggle",
					width = "full",
					set = setOption,
					get = getOption,
					arg = "SkipFightAnimations"
				};
				autoOpenRewards = {
					order = 10,
					name = L_AUTOOPENREWARDS,
					desc = L_AUTOOPENREWARDS_DESCRIPTION,
					type = "toggle",
					width = "full",
					set = setOption,
					get = getOption,
					arg = "AutoOpenRewards"
				};
				autoClose = {
					order = 15,
					name = L_AUTOCLOSE,
					desc = L_AUTOCLOSE_DESCRIPTION,
					type = "toggle",
					width = "full",
					set = setOption,
					get = getOption,
					arg = "AutoClose"
				};
			}
		};
		planner = {
			order = 20,
			type = "group",
			inline = true,
			name = L_PLANNER,
			args = {
				showFollowerCounterTotals = {
					order = 5,
					name = L_SHOWFOLLOWERCOUNTERTOTALS,
					desc = L_SHOWFOLLOWERCOUNTERTOTALS_DESCRIPTION,
					type = "toggle",
					width = "full",
					set = function(info, value)
						setOption(info, value);
						if ( GarrisonMissionFrame:IsShown() and GarrisonMissionFrame.FollowerTab:IsShown()) then
							if(value and not MissionMinderActiveFollowerCounters:IsShown() ) then
								MissionMinderActiveFollowerCounters:Show();
								if ( MissionMinder.ptr ) then
									GarrisonThreatCountersFrame:Hide();
								end
							elseif(not value and MissionMinderActiveFollowerCounters:IsShown()) then
								MissionMinderActiveFollowerCounters:Hide();
								if ( MissionMinder.ptr ) then
									GarrisonThreatCountersFrame:Show();
								end
							end
						end
					end,
					get = getOption,
					arg = "ShowFollowerCounterTotals"
				};
				ignoreXPGainTraits = {
					order = 10,
					name = L_IGNOREXPGAINTRAITS,
					desc = L_IGNOREXPGAINTRAITS_DESCRIPTION,
					type = "toggle",
					width = "full",
					set = setOption,
					get = getOption,
					arg = "IgnoreXPGainTraits"
				};
			}
		};
		--[===[@debug@
		spacer = {
			order = 9000,
			type = "description",
			name = " ",
			width = "double"
		};
		debugSpacer = {
			order = 9001,
			type = "description",
			name = " ",
			width = "normal"
		};
		debug = {
			order = 9002,
			name = L_DEBUG,
			type = "toggle",
			set = setOption,
			get = getOption,
			width = "half",
			arg = "Debug"
		};
		--@end-debug@]===]
	}
});
local configPane = LibStub("AceConfigDialog-3.0"):AddToBlizOptions(configName, addon.name);


local function showConfigPane()
	InterfaceOptionsFrame_OpenToCategory(configPane);

	-- #31: Blizzard options interface does not open pane on initial load
	if ( not configPane:IsShown() ) then
		InterfaceOptionsFrame_OpenToCategory(configPane);
	end
end


---------------------------------------------
-- MissionMinderConfigureButton MixIn
---------------------------------------------
MissionMinderConfigureButtonMixIn  = {}
function MissionMinderConfigureButtonMixIn:OnEnter()
	self.texture:SetAlpha(1.0)
	GameTooltip:SetOwner(self, "ANCHOR_RIGHT")
	GameTooltip:SetText(L_CONFIGURE_MISSIONMINDER)
	GameTooltip:Show()
end

function MissionMinderConfigureButtonMixIn:OnLeave()
	self.texture:SetAlpha(0.75)
	GameTooltip_Hide()
end

function MissionMinderConfigureButtonMixIn:OnMouseDown()
	self.texture:SetPoint("TOPLEFT", 1, -1)
end

function MissionMinderConfigureButtonMixIn:OnMouseUp()
	self.texture:SetPoint("TOPLEFT", 0, 0)
end

function MissionMinderConfigureButtonMixIn:OnClick()
	HideParentPanel(self)
	showConfigPane()
end

---------------------------------------------
-- INITIALIZE
---------------------------------------------
function module.initialize()
	--[===[@alpha@
	SLASH_MISSIONMINDER1 = "/missionminder";
	SlashCmdList["MISSIONMINDER"] = showConfigPane
	--@end-alpha@]===]
end

function module.register(missionFrame)
	-- Create MissionFrame.ConfigureButton
	CreateFrame("BUTTON", nil, missionFrame, "MissionMinderConfigureButtonTemplate")

	-- Create LandingPage.ConfigureButton, if not already created
	if ( not GarrisonLandingPage.ConfigureButton ) then
		CreateFrame("BUTTON", nil, GarrisonLandingPage, "MissionMinderConfigureButtonTemplate")
	end
end
